/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.servlet.bridge;

import org.xquark.bridge.*;
import java.util.*;
import java.sql.*;

public class XQBridgePool {

  static private XQBridgePool instance = new XQBridgePool();

  private HashMap urlMap = new HashMap();
  private String defaultUrl = null;
  private String defaultUser = null;
  private String defaultPassword = null;
  
  private XQBridgePool() {
      String[] DRIVER_LIST = { "com.mysql.jdbc.Driver", "com.microsoft.jdbc.sqlserver.SQLServerDriver", "com.sybase.jdbc2.jdbc.SybDriver", "oracle.jdbc.driver.OracleDriver" };
      for (int i = 0; i < DRIVER_LIST.length; i++) {
		try {
			Class.forName(DRIVER_LIST[i]);
		} catch (Exception e) {
		      // No driver found. 
		}
	}
  }

  public static XQBridgePool getXQBridgePool() {
    return instance;
  }
  
  public static XQBridgePool getXQBridgePool(String url, String user, String passwd) {
    instance.defaultUrl = url;
    instance.defaultUser = user;
    instance.defaultPassword = passwd;
    return instance;
  }
  
  public synchronized void clearPool(String url, String user) throws Exception {
    if (url == null || user == null) return;
    String key = url+":"+user;
    List bridgeList = (List)urlMap.remove(key);
    if (bridgeList != null) {
      Iterator it = bridgeList.iterator();
      while (it.hasNext()) {
	XQBridge bridge = (XQBridge)it.next();
	bridge.getXMLConnection().close();
	bridge.getConnection().close();
      }
    }
  }
  
  public synchronized XQBridge getXQBridge(String url, String user, String passwd) throws Exception {
    if (url == null || user == null || passwd == null) return null;
    String key = url+":"+user;
    List bridgeList = (List)urlMap.get(key);
    if (bridgeList == null) {
      bridgeList = new ArrayList();
      urlMap.put(key, bridgeList);
    }
    if (bridgeList.size() > 0)
      return (XQBridge)bridgeList.remove(bridgeList.size()-1);
    else {
      Connection jdbc = DriverManager.getConnection(url, user, passwd);
      return new XQBridge(jdbc);
    }
  }

  public synchronized void freeXQBridge(String url, String user, XQBridge bridge) throws Exception {
    if (url == null || user == null || bridge == null) return;
    String key = url+":"+user;
    List bridgeList = (List)urlMap.get(key);
    // This occurs only if the connections have been cleared; 
    if (bridgeList == null) return;
    bridgeList.add(bridge);
  }

  public synchronized void clearPool() throws Exception {
    clearPool(defaultUrl, defaultUser);
  }

  public synchronized XQBridge getXQBridge() throws Exception {
    return getXQBridge(defaultUrl, defaultUser, defaultPassword);
  }  

  public synchronized void freeXQBridge(XQBridge bridge) throws Exception {
    freeXQBridge(defaultUrl, defaultUser, bridge);
  }
}

