/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.servlet.util;

import java.io.File;
import java.io.FileFilter;


/**
 * FileFilter by file extension. File extension id not case-sensitive. 
 */
public class FileExtensionFilter implements FileFilter
{
	public String ext;
	
	
	/**
	 * @param ext The file extension, i.e., <b>.xml</b>
	 */
	public FileExtensionFilter(String ext)
	{
		this.ext = ext.toLowerCase();
	}
	
	
	/**
	 * @param ext The file extension, i.e., <b>.xml</b>
	 */
	public void setFileExtension(String ext)
	{
		this.ext = ext.toLowerCase();	
	}
		
	
	public boolean accept(File file)
	{
		if(file.isDirectory())
			return false;
		if(file.getName().toLowerCase().endsWith(ext))
			return true;
		return false;
	}
}

