/*
 * This file belongs to the XQuark distribution.
 * Copyright (C) 2003 Universite de Versailles Saint-Quentin.
 * 
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307.
 * You can also get it at http://www.gnu.org/licenses/lgpl.html
 * 
 * For more information on this software, see http://www.xquark.org.
 */

package org.xquark.servlet.util;

import java.util.HashMap;
import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;


/**
 * This class processes the "multipart/form-data" sent by html form.
 *  
 */
public class PostDataParser {
  static final int PAD_LENGTH = "-----------------------------".length();
	
  /**
   * The data to parse.
   */
  public StringBuffer data;
	
  public String queryString;
	
  HashMap parameterMap;
  /**
   * The http request identifier or the web browser identifier? 
   * This 12-digit Hexa number is used as field delimiter within the stream.
   */
  String RequestID;
	
	
  /**
   * @param data the StringBuffer containing the input data.
   */
  public PostDataParser(String data)
  {
    this.data = new StringBuffer(data);
    queryString = new String(data);
    this.parameterMap = new HashMap();
    parseRequestID();
    parseParameters();
  }
	
	
  public PostDataParser(HttpServletRequest request)
    throws IOException 
  {
    ServletInputStream in = request.getInputStream();
    this.data = new StringBuffer();
    int c;
    while((c=in.read()) != -1) this.data.append((char)c);
		
    queryString = new String(data);
    //PrintWriter out = new PrintWriter(new FileWriter("c:\\1.txt"));
    //out.print(queryString);
    //out.close();
    this.parameterMap = new HashMap();
    parseRequestID();
    parseParameters();
  }
	
	
  /**
   * Extracts the request ID from the input stream. It is a 12-digits hexa-dicimal number
   * preceeded by 28 '-' chars.  
   */
  void parseRequestID()
  {
    this.RequestID = data.substring(PAD_LENGTH, PAD_LENGTH+12);
    this.data = data.delete(0, PAD_LENGTH + this.RequestID.length());
  }
	 
	 
  private String getRequestID()
  {
    return this.RequestID; 
  }
	 
	 
  private void parseParameters()
  {
    String s = null;
		 
    while((s=extractParameter()) != null)
      parseParameter(s);		 
  }
	 
	 
  private String extractParameter()
  {
    int begin = 0;
    int end = 0;
		 
    begin = data.toString().indexOf("Content-Disposition: form-data; ") + "Content-Disposition: form-data; ".length();
    // 32 is the length of "Content-Disposition: form-data; "
		
		
    if(begin == 31) // 31 (=32-1) means the requested string not found.  
      return null;
		
    end = data.toString().indexOf(RequestID) - PAD_LENGTH;
    // 29 is the length of " ----------------------------" preceeding the Request ID ;
		
    String parameter = data.substring(begin, end);
		
    this.data = this.data.delete(0, end+PAD_LENGTH+this.RequestID.length()+1);
    // 43 is the length of " ----------------------------" plus the length of the Request ID
    return parameter;
  }
	 
	 
  private void parseParameter(String s)
  {
    if(s== null)
      return;
		 
    int begin = s.indexOf("\""); 
    int end = s.indexOf("\"", begin+1);
    //String name = s.substring(begin+1, end).trim();
    String name = s.substring(begin+1, end).trim();
    String value = s.substring(end+2, s.length()-1);
		
    if(isFileEntry(value))
      {
	this.parameterMap.put(name, parseUploadFile(value));	
      }
    else
      {
	this.parameterMap.put(name, value.trim());	
      }
  }
	 
  boolean isFileEntry(String data) 
  {
    return data.startsWith(" filename=\"");
  }
	 
	 
  private FileEntry parseUploadFile(String data)
  {
		
    int begin = " filename=\"".length();
    int end = data.indexOf("\"", begin);
			 
    String filename = data.substring(begin, end);
			
    begin = data.indexOf("\r\nContent-Type: ", end) + "\r\nContent-Type: ".length();
    end =  data.indexOf("\r\n", begin);
			
    String contentType = data.substring(begin, end);	
    String fileContent = data.substring(end+1, data.length()-1);

				
    /*
		
      System.out.println("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
      System.out.println("FILENAME="+filename);
      System.out.println("CONTENT-TYPE="+contentType);
      System.out.println("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");
			
      System.out.println("FILE=<"+fileContent+">=FILE");
      System.out.println("++++++++++++++++++++++++++++++++++++++++++++++++++++++++++");

    */
		
    return new FileEntry(filename, contentType, fileContent);
  }
	 
	 
	 
  /**
   * Get the value of an parameter. To conform the behaviour of HttpServletRequest, the
   * parameter names are case-sensitive.
   * 
   * @return the value of the parameter; a String object for string parameters and 
   * a FileEntry object if the parameter is a file upload.  
   */
  public String getParameter(String name) 
  {
    return (String) this.parameterMap.get(name); 
  }
	 
	 
  public String getFileContent(String name) 
  {
    return ((FileEntry)this.parameterMap.get(name)).getContent(); 
  }
	 
	 
  public FileEntry getFileEntry(String name) 
  {
    return (FileEntry) this.parameterMap.get(name); 
  }
	 
	 
  /**
   * Test program
   */
  public static void main(String args[])
    throws Exception 
  {
    StringBuffer data = new StringBuffer();
    FileReader in = new FileReader("post.txt");
    int c;
    while((c=in.read()) != -1)
      data.append((char)c);
    PostDataParser p = new PostDataParser(data.toString());
    //System.out.println("");
  }
}
